from sympy import symbols, diff, integrate, Eq, solve
import numpy as np
import matplotlib.pyplot as plt

class ScientificCalculator:
    def solve_equation(self, expression):
        try:
            x = symbols('x')
            eq = Eq(expression, 0)
            sol = solve(eq, x)
            return sol
        except Exception as e:
            print("Error: Invalid equation.")
            return None

    def differentiate_expression(self, expression):
        try:
            x = symbols('x')
            derivative = diff(expression, x)
            return derivative
        except Exception as e:
            print("Error: Invalid mathematical expression.")
            return None

    def integrate_expression(self, expression):
        try:
            x = symbols('x')
            integral = integrate(expression, x)
            return integral
        except Exception as e:
            print("Error: Invalid mathematical expression.")
            return None

    def perform_matrix_operation(self, operation, matrix1, matrix2):
        try:
            if operation == "addition":
                result = np.add(matrix1, matrix2)
            elif operation == "subtraction":
                result = np.subtract(matrix1, matrix2)
            elif operation == "multiplication":
                result = np.dot(matrix1, matrix2)
            elif operation == "determinant":
                result = np.linalg.det(matrix1)
            else:
                result = None
            return result
        except Exception as e:
            print("Error: Invalid matrix operation.")
            return None

    def plot_graph(self, expression, start, end):
        try:
            x = np.linspace(start, end, 100)
            y = [expression.subs('x', xi) for xi in x]
            plt.plot(x, y)
            plt.xlabel('x')
            plt.ylabel('y')
            plt.title('Graph Plot')
            plt.grid(True)
            plt.show()
        except Exception as e:
            print("Error: Invalid mathematical expression for graph plotting.")

# Example usage
calculator = ScientificCalculator()

# Equation Solving
equation = input("Enter an equation: ")
solution = calculator.solve_equation(equation)
if solution:
    print("Solution:", solution)

# Symbolic Differentiation
expression = input("Enter a mathematical expression for differentiation: ")
derivative = calculator.differentiate_expression(expression)
if derivative:
    print("Derivative:", derivative)

# Symbolic Integration
expression = input("Enter a mathematical expression for integration: ")
integral = calculator.integrate_expression(expression)
if integral:
    print("Integral:", integral)

# Matrix Operations
matrix1 = np.array([[1, 2], [3, 4]])
matrix2 = np.array([[5, 6], [7, 8]])
operation = input("Enter matrix operation (addition, subtraction, multiplication, determinant): ")
result = calculator.perform_matrix_operation(operation, matrix1, matrix2)
if result is not None:
    print("Result:", result)

# Graph Plotting
expression = input("Enter a mathematical expression for graph plotting: ")
start, end = input("Enter x range (start end): ").split()
try:
    start = float(start)
    end = float(end)
    calculator.plot_graph(expression, start, end)
except ValueError:
    print("Error: Invalid x range values.")
